
########## DATASET GENERATION ###########

#Constants
N_sim <- 1000 #number of simulations
N_obs <- 400 #number of observations per simulation

#get indexes of treatment and control 
g_treat <- which(newdata2$treat ==1 ) 
g_control <- which(newdata2$treat == 0)


set.seed(100) 

#create treatment indicator
treat_ind <- rbinom(n = N_obs, size = 1, prob = 0.75) 

#create index
dt <- c(sample(g_treat, size = sum(treat_ind == 1), replace = TRUE), sample(g_control, size = sum(treat_ind == 0), replace = TRUE)) 

#Append data from original dataset based in index
df_0.75 <- data.frame( cd40 = newdata2$cd40[dt],        cd420 = newdata2$cd420[dt],      cd80 = newdata2$cd80[dt],       
                                age = newdata2$age[dt],          wtkg = newdata2$wtkg[dt],        karnof = newdata2$karnof[dt],   
                                hemo = as.numeric(levels(newdata2$hemo[dt]))[newdata2$hemo[dt]],   
                                homo = as.numeric(levels(newdata2$homo[dt]))[newdata2$homo[dt]], 
                                drugs = as.numeric(levels(newdata2$drugs[dt]))[newdata2$drugs[dt]],     
                                race = as.numeric(levels(newdata2$race[dt]))[newdata2$race[dt]],     
                                gender = as.numeric(levels(newdata2$gender[dt]))[newdata2$gender[dt]],
                                str2 = as.numeric(levels(newdata2$str2[dt]))[newdata2$str2[dt]],  
                                symptom = as.numeric(levels(newdata2$symptom[dt]))[newdata2$symptom[dt]], 
                                treat = as.numeric(levels(newdata2$treat[dt]))[newdata2$treat[dt]],      which = dt,     
                                nsim = 1) 

#begin simulation
for (i in 2:N_sim) {
  
  #create treatment indicator
  treat_ind <- rbinom(n = N_obs, size = 1, prob = 0.75)
  
  #create index
  dt <- c(sample(g_treat, size = sum(treat_ind == 1), replace = TRUE), sample(g_control, size = sum(treat_ind == 0), replace = TRUE))
 
  #Append data from original dataset based in index
  df_0.75 <- df_0.75 %>% add_row( cd40 = newdata2$cd40[dt],        cd420 = newdata2$cd420[dt],      cd80 = newdata2$cd80[dt],       
                                  age = newdata2$age[dt],          wtkg = newdata2$wtkg[dt],        karnof = newdata2$karnof[dt],   
                                  hemo = as.numeric(levels(newdata2$hemo[dt]))[newdata2$hemo[dt]],   
                                  homo = as.numeric(levels(newdata2$homo[dt]))[newdata2$homo[dt]], 
                                  drugs = as.numeric(levels(newdata2$drugs[dt]))[newdata2$drugs[dt]],     
                                  race = as.numeric(levels(newdata2$race[dt]))[newdata2$race[dt]],     
                                  gender = as.numeric(levels(newdata2$gender[dt]))[newdata2$gender[dt]],
                                  str2 = as.numeric(levels(newdata2$str2[dt]))[newdata2$str2[dt]],  
                                  symptom = as.numeric(levels(newdata2$symptom[dt]))[newdata2$symptom[dt]], 
                                  treat = as.numeric(levels(newdata2$treat[dt]))[newdata2$treat[dt]],      which = dt,     
                                  nsim = i) 
  
}


#proportion = 0.5
set.seed(100)
treat_ind <- rbinom(n = N_obs, size = 1, prob = 0.50)
dt <- c(sample(g_treat, size = sum(treat_ind == 1), replace = TRUE), sample(g_control, size = sum(treat_ind == 0), replace = TRUE))


df_0.50 <- cbind(newdata2[dt,], which = dt, nsim = 1)

for (i in 2:N_sim) {
  treat_ind <- rbinom(n = N_obs, size = 1, prob = 0.50)
  dt <- c(sample(g_treat, size = sum(treat_ind == 1), replace = TRUE), sample(g_control, size = sum(treat_ind == 0), replace = TRUE))
  
  df_0.50 <- df_0.50 %>% add_row( cd40 = newdata2$cd40[dt],        cd420 = newdata2$cd420[dt],      cd80 = newdata2$cd80[dt],       
                                  age = newdata2$age[dt],          wtkg = newdata2$wtkg[dt],        karnof = newdata2$karnof[dt],   
                                  hemo = newdata2$hemo[dt],        homo = newdata2$homo[dt],        drugs = newdata2$drugs[dt],     
                                  race = newdata2$race[dt],        gender = newdata2$gender[dt],    str2 = newdata2$str2[dt],       
                                  symptom = newdata2$symptom[dt],  treat = newdata2$treat[dt],      which = dt,     
                                  nsim = i) 
  
}

format(object.size(df_0.50), units = 'Mb')
sum(df_0.75$treat)


