#FUNCTIONS

#calculate empirical influence function estimates for difference in means 
infl.fun <- function (response, treatment){
  m0 <- mean(response[treatment == 0])
  m1 <- mean(response[treatment == 1])
  p  <- length(treatment[treatment == 1])/length(treatment)
  s  <- (treatment*(response - m1))/p - ((1-treatment)*(response - m0))/(1-p)
  return(s)  
}

infl.fun2 <- function (response, treatment){
  p  <- length(treatment[treatment == 1])/length(treatment)
  m0 <- sum(( (1-treatment) * response) / ( (1-p)* length(treatment)))
  m1 <- sum(( treatment * response) / ( p * length(treatment)))
  s  <- (treatment*(response - m1))/p - ((1-treatment)*(response - m0))/(1-p)
  return(s)  
}


#Estimete the augmented estimator, standard error and relative efficiency 
theta_aug <- function( m_control, m_treat, data)
{
  pred_c <- predict(m_control, data[,-c(2,14)])
  pred_t <- predict(m_treat, data[,-c(2,14)])
  ax<-0
  if(class(m_control)== "SuperLearner"){
    pred_t <-pred_t$pred
    pred_c <-pred_c$pred
    ax<-pred_t-pred_c
  }

  n0 <- length(data$cd420[data$treat ==0])
  n1 <- length(data$cd420[data$treat ==1])
  prop <- n1/(n0+n1)
  spred_c <- (1/n0)*pred_c
  spred_t <- (1/n1)*pred_t
  meany_c <- mean(data$cd420[data$treat == 0])
  meany_t <- mean(data$cd420[data$treat == 1])
  meanpred_c <- mean(pred_c)
  meanpred_t <- mean(pred_t)
  t0 <- meany_c - meanpred_c
  t1 <- meany_t - meany_t
  treat_indicator <- as.numeric(levels(data$treat))[data$treat]
  #treat_indicator <- data$treat
  theta_est <- meany_t - meany_c - sum( (treat_indicator - prop) * (spred_c + spred_t) )
  theta_var <- sum((( (1/n1)*treat_indicator - (1/n0)*(1-treat_indicator) ) * data$cd420 - 1/(n0+n1)*theta_est - (treat_indicator - prop)*( spred_c + spred_t ) - (treat_indicator - prop)*(((1/n0)*t0) + ((1/n1)*t1)) )^2 )
  
  seun <- 6.760197 # sqrt(sd(cd420[treat == 1])^2/length(cd420[treat == 1]) + sd(cd420[treat == 0])^2/length(cd420[treat == 0]))  
  rel_ef<- (seun^2)/theta_var
  ret <- list(theta_est, sqrt(theta_var), (rel_ef))
  names(ret) <- c('theta', 'SE theta', 'Rel Eff')
  return(ret)
}


#estimate variance of augmented theta 
aug.var <-function (response, data){
  treatment<- data$treat
  p <- length(treatment[treatment == 1])/length(treatment)
  n <- length(treatment)
  s <- sum( (infl.fun(response, treatment) - (treatment-p)*aug.fun(...))^2)/n
  return(s)  
}


#
theta_aug2 <- function( m_control, m_treat, data)
{
  pred_c <- predict(m_control, data[,-c(1,2)])
  
  pred_t <- predict(m_treatment, data[,-c(1,2)])
  ax<-0
  if(class(m_control)== "SuperLearner"){
    pred_t <-pred_t$pred
    pred_c <-pred_c$pred
    ax<-pred_t-pred_c
  }
  n0 <- length(data$cd420[data$treat ==0])
  n1 <- length(data$cd420[data$treat ==1])
  prop <- n1/(n0+n1)
  spred_c <- (1/n0)*pred_c
  spred_t <- (1/n1)*pred_t
  meany_c <- mean(data$cd420[data$treat == 0])
  meany_t <- mean(data$cd420[data$treat == 1])
  
  meanpred_c <- mean(pred_c)
  meanpred_t <- mean(pred_t)
  t0 <- meany_c - meanpred_c
  t1 <- meany_t - meany_t
  treat_indicator <- as.numeric(data$treat)
  theta_est <- meany_t - meany_c - sum( (treat_indicator - prop) * (spred_c + spred_t) )
  theta_var <- sum((( (1/n1)*treat_indicator - (1/n0)*(1-treat_indicator) ) * data$cd420 - 1/(n0+n1)*theta_est - (treat_indicator - prop)*( spred_c + spred_t ) - (treat_indicator - prop)*(((1/n0)*t0) + ((1/n1)*t1)) )^2 )
    
  seun <- 6.760197 # sqrt(sd(cd420[treat == 1])^2/length(cd420[treat == 1]) + sd(cd420[treat == 0])^2/length(cd420[treat == 0]))  
  rel_ef<- (seun^2) / theta_var
  ret <- list(theta_est, sqrt(theta_var), (rel_ef))
  names(ret) <- c('theta', 'SE theta', 'Rel eff')
  return(ret)
}



#Fit Super learner models for specific wrapper - all linear terms
model_fitting <-function(data, wrapper = 'SL.glm')
{
  set.seed(1)
  m_control <- SuperLearner(data$cd420[data$treat == 0], data[data$treat == 0, -c(2,14)],  SL.library = wrapper )
  set.seed(1)
  m_treatment <- SuperLearner(data$cd420[data$treat == 1], data[data$treat == 1, -c(2,14)],  SL.library = wrapper )

  theta <- theta_aug(m_control, m_treatment, data)
  return(theta)
}


#Fit Super learner models for specific wrapper - full specified data
model_fitting2 <-function(data, wrapper = 'SL.glm')
{
  mm <- model.matrix(cd420~.^2 + I(cd40^2) + I(cd80^2) + I(age^2) + I(wtkg^2) + I(karnof^2), data = data[,-14])
  set.seed(1)
  m_control <- SuperLearner(data$cd420[data$treat == 0], mm[data$treat == 0, -c(1,14)],  SL.library = wrapper )
  set.seed(1)
  m_treatment <- SuperLearner(data$cd420[data$treat == 1], mm[data$treat == 1, -c(1,14)],  SL.library = wrapper )
  
  theta <- theta_aug2(m_control, m_treatment, data.frame(cbind(cd420, treat, mm[,-c(1)])))
  return(theta)
}


##Super learner wrappers - GLMNETS
SL.rf.glmnet_lasso1 = function(...) {
  SL.glmnet(..., alpha = 1, loss = 'mse',  nfolds = 10  )
}

SL.rf.glmnet_lasso2 = function(...) {
  SL.glmnet(..., alpha = 1, loss = 'mse',  nfolds = 20  )
}

SL.rf.glmnet_ridge1 = function(...) {
  SL.glmnet(..., alpha = 0,  loss = 'mse',  nfolds = 10   )
}

SL.rf.glmnet_ridge2 = function(...) {
  SL.glmnet(..., alpha = 0,  loss = 'mse',  nfolds = 20  )
}




#Super learner wrappers - Random Forest
SL.rf.rforest.4.100 = function(...) {
  SL.randomForest(..., mtry = 4 , ntree =100 )
}

SL.rf.rforest.4.500 = function(...) {
  SL.randomForest(..., mtry = 4 , ntree =500 )
}

SL.rf.rforest.4.1000 = function(...) {
  SL.randomForest(..., mtry = 4 , ntree =1000 )
}


SL.rf.rforest.3.100 = function(...) {
  SL.randomForest(..., mtry = 3 , ntree =100 )
}

SL.rf.rforest.3.500 = function(...) {
  SL.randomForest(..., mtry = 3 , ntree =500 )
}

SL.rf.rforest.3.1000 = function(...) {
  SL.randomForest(..., mtry = 3 , ntree =1000 )
}


#Fitt models for simulated data
simul<- function (N_sim  = 100, N_obs = 400, method = 'all', data, theta = 46.8105, se_theta = 6.760197){
  
  if (method == "lm"){
    f_wrapper <- "SL.lm"
  } else if ( method == "lasso"){
    f_wrapper <- "SL.rf.glmnet_lasso1"
  } else if (method == 'ridge') {
    f_wrapper <- "SL.rf.glmnet_ridge1"
  } else if (method == "rpart") {
    f_wrapper <- "SL.rpart"
  } else if (method == "forest") {
    f_wrapper <- "SL.rf.rforest.4.100"
  } else {
    f_wrapper <- c("SL.lm", "SL.rf.glmnet_lasso1", "SL.rf.glmnet_ridge1" , "SL.rpart", "SL.rf.rforest.4.100" ) 
  }
  
  sim_measures <- data.frame(theta_stim = numeric(N_sim), se_stim = numeric(N_sim), n1_sim = numeric(N_sim), n0_sim = numeric(N_sim))
  
  set.seed(100)
  for (i in 1:N_sim) {
    
    temp <- model_fitting(data[data$nsim == i, -c( 15,16)], wrapper = f_wrapper )
    
    n1_sim <- sum(data[(data$nsim == i),]$treat==1)
    n0_sim <- N_obs - n1_sim
    sim_measures[i, ] <- cbind(temp$theta, temp$`SE theta`, n1_sim, n0_sim)
  }
  
  
  bias <-mean(sim_measures$theta_stim) - theta
  se_bias <- sqrt ( sum((sim_measures$theta_stim - mean(sim_measures$theta_stim))^2)/(N_sim* (N_sim - 1 )))
  B_se<- sqrt ( sum((sim_measures$theta_stim - mean(sim_measures$theta_stim))^2)/( (N_sim - 1 )))
  
  
  CI <- data.frame(theta_low = (sim_measures$theta_stim + qnorm(0.025) *  B_se), 
                   theta_high = (sim_measures$theta_stim + qnorm(0.975)* B_se) )
  
  #coverage probability 
  cp <- mean((theta >= CI$theta_low & theta <= CI$theta_high)) 
  
  ret <- list(bias, se_bias, B_se, cp)
  names(ret) <- c('Bias', 'SE Bias',"EMP SE", "CP")
  return(ret)
  
}